/*
  Copyright (C) 2011  Philipp Kaelin

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * \author  Philipp Kaelin
 * \date    created:  2012/01/18 <br>
 *          modified: 2012/01/18
 * \brief   Implements all the states of the finite state machine
 * \defgroup ApplicationState States of FSM
 */

#ifndef APPLICATIONSTATE_H_
#define APPLICATIONSTATE_H_

#include "State.h"

 /**
  * \ingroup ApplicationState
  * \brief Bereitzustand
  */
class StateBereit : public State {
  public:
    void entryAction();
    void exitAction();
    void tickAction();
};

 /**
  * \ingroup ApplicationState
  * \brief Toast wird gerade eingezogen
  */
class StateToastEinziehen : public State {
  public:
    void entryAction();
    void exitAction();
    void tickAction();
};

 /**
  * \ingroup ApplicationState
  * \brief Toast wird gerade ausgeworfen
  */
class StateToastAuswerfen : public State {
  public:
    void entryAction();
    void exitAction();
    void tickAction();
};

 /**
  * \ingroup ApplicationState
  * \brief Dem Toast wird gerade so richtig eingeheizt
  */
class StateToasten : public State {
    void entryAction();
    void exitAction();
    void tickAction();
  private:
    int toastTime;
};

 /**
  * \ingroup ApplicationState
  * \brief Beim einziehen oder auswerfen ist ein Fehler aufgetreten
  */
class StateFehler : public State {
  public:
    void entryAction();
    void exitAction();
    void tickAction();
};

#endif /* APPLICATIONSTATE_H_ */
