/*
  Copyright (C) 2011  Philipp Kaelin

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "Toaster.h"
#include "ApplicationState.h"

using namespace Toaster;

static StateBereit          bereit;
static StateToastEinziehen  einziehen;
static StateToastAuswerfen  auswerfen;
static StateToasten         toasten;
static StateFehler          fehler;

State* const                initState = &bereit;

Toaster::Transition fsmApp[] = {
// actualState                     MaxTimeBeforeTransition
//             Event                               Next State
  {&bereit,    TasterStart,        0,              &einziehen },

  {&einziehen, EndschalterDrinnen, 0,              &toasten   },
  {&einziehen, Tick,               Seconds(15),    &fehler    },

  {&toasten,   TasterFertig,       0,              &auswerfen },
  {&toasten,   Tick,               Seconds(45),    &auswerfen },

  {&auswerfen, EndschalterLeer,    0,              &bereit    },
  {&auswerfen, Tick,               Seconds(15),    &fehler    },
};


//#############################################################################
// ---------------------- DO NOT CHANGE BELOW THIS LINE -----------------------
//#############################################################################
uint16_t timeInState = 0;
State* currentState;

volatile int evQueueSize; //
volatile int evQueueHead;  // index begin data
volatile int evQueueTail;  // index stop data
volatile Event evQueue[EventQueueSize];

//-----------------------------------------------------------------------------
inline bool evQueueIsEmpty() { return (evQueueHead == evQueueTail); }

//-----------------------------------------------------------------------------
inline bool evQueueIsFull() { return ( ((evQueueTail + 1) & (evQueueSize - 1)) == evQueueHead ); }

//-----------------------------------------------------------------------------
void Toaster::init() {
  evQueueSize = EventQueueSize;
  evQueueHead = 0;
  evQueueTail= 0;

  currentState = initState; //const_cast<State*>(initState);
  currentState->entryAction();
}

//-----------------------------------------------------------------------------
void Toaster::sendEvent(Event e) {
  if (!evQueueIsFull()) {
    evQueue[evQueueTail] = e;
    evQueueTail = (evQueueTail + 1) & (evQueueSize - 1);
  }
}

//-----------------------------------------------------------------------------
bool Toaster::process() {
  if (evQueueIsEmpty()) {
    return false;
  }
  Event e = evQueue[evQueueHead];
  evQueueHead = (evQueueHead + 1) & (evQueueSize - 1);

  for ( uint8_t i=0; i<sizeof(fsmApp)/sizeof(Transition); i++ ) {
    if (   ( (currentState == fsmApp[i].currentState) && (e == fsmApp[i].event) )
        || (fsmApp[i].event == 0)   )
    {
      if (e == Tick) {
        // System Tick
        fsmApp[i].currentState->tickAction();

        timeInState++;
        if (   (fsmApp[i].maxTimeBeforeTransition != 0)
            && (timeInState < fsmApp[i].maxTimeBeforeTransition) )
        {
          // Normal Tick
          fsmApp[i].currentState->tickAction();
        } else {
          // Force Transition because of an Timeout
          fsmApp[i].currentState->exitAction();
          currentState = fsmApp[i].nextState;
          currentState->entryAction();
          timeInState = 0;
        }
      } else {
        timeInState = 0;
        fsmApp[i].currentState->exitAction();
        currentState = fsmApp[i].nextState;
        currentState->entryAction();
      }
      break;
    }
  }
  return true;
}
