/*
 Copyright (C) 2012  Philipp Kaelin

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "Toaster.h"
#include "Hardware.h"

/************************************************************
 * Angeschlossene Hardware:
 *
 * Tasten:      Start          PD.0     fallende Flanke
 *              Fertig:        PD.1     fallende Flanke
 *
 * Endschalter: Drinnen        PD.2     fallende Flanke
 *              Leer:          PD.3     fallende Flanke
 *
 * LEDs         Ready          PB.1
 *              Heating        PB.2
 *              Error          PB.3
 * 
 * Motor:       Linkslauf:     PA.1
 *              Rechtslauf     PA.2
 * 
 * Heizung:                    PC.1
 *
 ************************************************************/



//-----------------------------------------------------------------------------
void initOneSecondTimer() {
  // Prescaler 1024
  TCCR0 = (1 << CS00)
        | (1 << CS01)
        | (1 << CS02);
 
  // Overflow Interrupt erlauben
  TIMSK |= (1 << TOIE0);
}

//-----------------------------------------------------------------------------
ISR(TIMER0_OVF_vect) {
  Toaster::sendEvent(Toaster::Tick);
}

//-----------------------------------------------------------------------------
void initInterrupt() {
  // falling edge
  EICRA = (1 << ISC01) | (1 << ISC11) | (1 << ISC21) | (1 << ISC31);

  // enable interrupt
  EIMSK = (1 << INT0) | (1 << INT1) | (1 << INT2) | (1 << INT3);
}

//-----------------------------------------------------------------------------
ISR(INT0_vect) {
  Toaster::sendEvent(Toaster::TasterStart);
}

//-----------------------------------------------------------------------------
ISR(INT1_vect) {
  Toaster::sendEvent(Toaster::TasterFertig);
}

//-----------------------------------------------------------------------------
ISR(INT2_vect) {
  Toaster::sendEvent(Toaster::EndschalterDrinnen);
}

//-----------------------------------------------------------------------------
ISR(INT3_vect) {
  Toaster::sendEvent(Toaster::EndschalterLeer);
}

//-----------------------------------------------------------------------------
int main() {
  // Init Hardware
  Motor::init();
  Led::init();
  Heating::init();
 
  // Init FSM
  Toaster::init();


  initOneSecondTimer();
  initInterrupt();

  // Global Interrupts aktivieren
  sei();

  while (1) {
    Toaster::process();
  }

  return 0;
}

